#include <errno.h>
#include <stdio.h>
#include <net/bsdlib.h>
#include <kernel.h>
#include <net/socket.h>
#include <at_cmd.h>
#include <lte_lc.h>

#define PORT 40000
#define IPADDR "192.168.1.105"

#include <logging/log.h>

/* Initialize module-specific magic state (once per module name): */
LOG_MODULE_REGISTER();

int clientInit()
{
    int fd, ret;
    struct addrinfo *p; 
    struct sockaddr_in bind_addr;
    char ipstr[INET_ADDRSTRLEN];
    struct addrinfo *result;
    struct addrinfo hints = {
            .ai_family = AF_INET,
            .ai_socktype = SOCK_STREAM
    };

    ret = getaddrinfo("www.google.com", NULL, &hints, &result);
    if (ret) 
    {
        printk("ERROR: getaddrinfo failed %d\n", ret);
    } 
    else{
        for (p = result; p != NULL; p = p->ai_next) {
            struct in_addr *addr;
            if (p->ai_family == AF_INET) {
            struct sockaddr_in *ipv = (struct sockaddr_in *)p->ai_addr;
            addr = &(ipv->sin_addr);
            } 
            else {
                struct sockaddr_in6 *ipv6 = (struct sockaddr_in6 *)p->ai_addr;
                addr = (struct in_addr *)&(ipv6->sin6_addr);
            }
            inet_ntop(p->ai_family, addr, ipstr, sizeof ipstr);
        }

        printf("Resolved Address: %s\n", ipstr);
    }

    fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (fd < 0) {
        printk("error: socket: %d\n", errno);
        return -1;
    } 
    else {
        printk("Socket successfully created!\n");
    }

    bind_addr.sin_family = AF_INET;
    bind_addr.sin_port = htons(PORT);
    inet_pton(AF_INET, IPADDR, &bind_addr.sin_addr.s_addr);

    ret = connect(fd, (struct sockaddr*)&bind_addr, sizeof(bind_addr));
    if (ret < 0) {
        printk("ERROR CONNECTING RET: %d ERRNO: %d\n", ret, errno);
        return -1;
    }

    if (ret < 0) {
        printk("Connect() failed: %d\n", ret);
        return -1;
    }

    char buf[] = "Helloww from nRF91!\n";

    int out_len = send(fd, buf, sizeof(buf), 0);
    if (out_len < 0) {
        printk("error: send: %d\n", errno);
    }

    close(fd);

    return 0;
}

static void modem_configure(void)
{
#if defined(CONFIG_LTE_LINK_CONTROL)
	if (IS_ENABLED(CONFIG_LTE_AUTO_INIT_AND_CONNECT)) {
		/* Do nothing, modem is already turned on
		 * and connected.
		 */
	} else {
		int err;

		printk("LTE Link Connecting ...\n");
		err = lte_lc_init_and_connect();
		__ASSERT(err == 0, "LTE link could not be established.");
		printk("LTE Link Connected!\n");
	}
#endif
}

void serverInit()
{
    int serv;
    struct sockaddr_in bind_addr;
    static int counter;

    serv = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

    if (serv < 0) {
            printf("error: socket: %d\n", errno);
            exit(1);
    }

    bind_addr.sin_family = AF_INET;
    bind_addr.sin_addr.s_addr = htonl(INADDR_ANY);
    bind_addr.sin_port = htons(PORT);

    if (bind(serv, (struct sockaddr *)&bind_addr, sizeof(bind_addr)) < 0) {
            printf("error: bind: %d\n", errno);
            exit(1);
    }

    if (listen(serv, 5) < 0) {
            printf("error: listen: %d\n", errno);
            exit(1);
    }

    printf("Single-threaded TCP echo server waits for a connection on port %d...\n", PORT);

    while (1) {
        struct sockaddr_in client_addr;
        socklen_t client_addr_len = sizeof(client_addr);
        char addr_str[32];
        int client = accept(serv, (struct sockaddr *)&client_addr, &client_addr_len);

        if (client < 0) {
              printf("error: accept: %d\n", errno);
              continue;
        }

        inet_ntop(client_addr.sin_family, &client_addr.sin_addr,
                  addr_str, sizeof(addr_str));
        printf("Connection #%d from %s\n", counter++, addr_str);

        while (1) {
            char buf[128], *p;
            int len = recv(client, buf, sizeof(buf), 0);
            int out_len;

            if (len <= 0) {
                  if (len < 0) {
                          printf("error: recv: %d\n", errno);
                  }
                  break;
            }

            p = buf;
            do {
                  out_len = send(client, p, len, 0);
                  if (out_len < 0) {
                          printf("error: send: %d\n", errno);
                          goto error;
                  }
                  p += out_len;
                  len -= out_len;
            } while (len);
        }
    }
error:
    close(serv);
}

int main(void) 
{

    printk("Modem Configure\n");
    modem_configure();

    clientInit();

    serverInit();

    return 0;
}